@@ -1,6 +1,7 @@
 # coding=utf-8
 import datetime
 import functools
+import hashlib
 import json
 import sys
 import threading
@@ -8,7 +9,7 @@ import traceback
 from copy import copy
 
 from django.contrib.auth.models import User, Group, Permission
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist, MultipleObjectsReturned
 from django.db.models import Q
 from django.db.models.base import ModelBase
 from django.db.transaction import atomic
@@ -70,6 +71,8 @@ class Node:
         self._conditions = []
         self._next = None
         self._wait = None
+        self._skipif = []
+        self._leave = False
         self.description = kwargs.pop('description', '')
 
     @staticmethod
@@ -112,6 +115,7 @@ class Node:
                 locked=0,
             ),
         )
+        self._leave = False
         if task.locked > 0:
             return None
         task.locked += 1
@@ -138,6 +142,7 @@ class Node:
         if task is not None:
             task.finished = now()
             self.release_lock(task)
+        if task is not None or self._leave:
             if self._next is None:
                 next_node = self._automation._iter[self]
             else:
@@ -167,8 +172,21 @@ class Node:
         self._automation._db.save()
         return self.release_lock(task)
 
+    @on_execution_path
+    def skip_handler(self, task: models.AutomationTaskModel):
+        for item in self._skipif:
+            if self.eval(item, task):
+                task.finished = now()
+                task.message = "skipped"
+                self.release_lock(task)
+                self._leave = True
+                return None
+        return task
+
     def execute(self, task: models.AutomationTaskModel):
-        return self.when_handler(self.wait_handler(task))
+        return self.when_handler(
+            self.wait_handler(
+                self.skip_handler(task)))
 
     def Next(self, next_node):
         if self._next is not None:
@@ -192,6 +210,10 @@ class Node:
         self._wait = lambda x: x.created + self.eval(timedelta, x)
         return self
 
+    def SkipIf(self, condition):
+        self._skipif.append(condition)
+        return self
+
     def __repr__(self):
         return f"<{f'{self._name}: ' if self._automation else ''}" \
                f"{self._automation if self._automation else 'unbound'} {self.__class__.__name__} node>"
@@ -580,31 +602,28 @@ class Automation:
                 nm_c = copy(name)
                 setattr(self.__class__, name,  # Replace property by get_model_instance
                         property(lambda slf: slf.get_model_instance(at_c, nm_c), self))
-                if name not in kwargs:
-                    kwargs[name] = None
-                elif not isinstance(kwargs[name], int):  # Convert instance to id
+                if name in kwargs and not isinstance(kwargs[name], int):  # Convert instance to id
                     kwargs[name] = kwargs[name].id
         self._iter[prev] = None  # Last item
         autorun = kwargs.pop("autorun", True)
         if 'automation' in kwargs:
-            self._db = kwargs.pop('automation')
             autorun = False
-            assert isinstance(self._db, models.AutomationModel), \
-                "automation= parameter needs to be AutomationModel instance"
+            if isinstance(kwargs.get("automation"), models.AutomationModel):
+                self._db = kwargs.pop('automation')
+            else:
+                self._db = self.model_class.objects.get(
+                    key=kwargs.pop('automation'),
+                )
+            assert self._db.automation_class == self.get_automation_class_name(), \
+                f"Wrong automation class: expected {self.get_automation_class_name()} " \
+                f"got {self._db.automation_class}"
             assert not kwargs, "Too many arguments for automation %s. " \
-                               "If 'automation' is given, no parameters allowed" % self.__clas__.__name__
+                               "If 'automation' is given, no parameters allowed" % self.__class__.__name__
         elif 'automation_id' in kwargs:  # Attach to automation in DB
-            self._create_model_properties(kwargs)
-            self._db, _ = self.model_class.objects.get_or_create(
-                id=kwargs.pop('automation_id'),
-                defaults=dict(
-                    automation_class=self.get_automation_class_name(),
-                    finished=False,
-                    data=kwargs,
-                ))
+            self._db = self.model_class.objects.get(id=kwargs.pop('automation_id'))
             autorun = False
             assert not kwargs, "Too many arguments for automation %s. " \
-                               "If 'automation' is given, no parameters allowed" % self.__clas__.__name__
+                               "If 'automation_id' is given, no parameters allowed" % self.__class__.__name__
         elif self.unique is True:  # Create or get singleton in DB
             self._db, created = self.model_class.objects.get_or_create(
                 automation_class=self.get_automation_class_name(),
@@ -615,9 +634,9 @@ class Automation:
                 self._db.save()
             else:
                 assert not kwargs, "Too many arguments for automation %s. " \
-                                   "If 'automation' is given, no parameters allowed" % self.__clas__.__name__
+                                   "If 'automation' is given, no parameters allowed" % self.__class__.__name__
         elif self.unique:
-            assert isinstance(self.unique, (list, tuple)), ".singleton can be bool, list, tuple or None"
+            assert isinstance(self.unique, (list, tuple)), ".unique can be bool, list, tuple or None"
             for key in self.unique:
                 assert key not in ("automation", "automation_id", "autorun"), \
                     f"'{key}' cannot be parameter to distinguish unique automations. Chose a different name."
@@ -718,15 +737,16 @@ class Automation:
         return f"Automations {cls.__name__}"
 
     def finished(self):
-        return self._db.finished
+        return self._db is not None and self._db.finished
 
     def send_message(self, message, token, data=None):
         """RECEIVES message and dispatches it within the class
         Called send_message so that sending a message to an automation
         is `automation.send_message(...)"""
         if self.__class__.satisfies_data_requirements(message, data):
-            method = getattr(self, "receive_"+message)
-            return method(token, data)
+            if not self.finished():
+                method = getattr(self, "receive_"+message)
+                return method(token, data)
         return None
 
     @classmethod
@@ -752,12 +772,19 @@ class Automation:
         self._db.delete()
         self._db = None
 
+    def get_key(self):
+        return self._db.get_key()
 
     @classmethod
     def dispatch_message(cls, automation, message, token, data):
-        if cls.satisfies_data_requirements(message, data):
-            if isinstance(automation, int):
-                automation = cls(automation_id=automation)
+        if cls.satisfies_data_requirements(message, data) and automation is not None:
+            try:
+                if isinstance(automation, int):
+                    automation = cls(automation_id=automation)
+                elif isinstance(automation, (str, models.AutomationModel)):
+                    automation = cls(automation=automation)
+            except (ObjectDoesNotExist, MultipleObjectsReturned):
+                return None
             assert isinstance(automation, cls), f"Wrong class to dispatch message: " \
                                                 f"{automation.__class__.__name__} found, " \
                                                 f"{cls.__name__} expected"
@@ -787,8 +814,10 @@ class Automation:
                 for param in cls.unique:
                     if param in accessor:
                         kwargs[param] = accessor.get(param)
-            instance = cls(autorun=False, **kwargs)
-            instance.send_message(message, token, data)
+            instance = cls(autorun=False, **kwargs)      # Create
+            instance.send_message(message, token, data)  # Allow message to be processes before ..
+            if not instance.finished():
+                instance.run()                           # ... automation is run
             return instance
         return None
 
