@@ -189,6 +189,10 @@ class FormTestCase(TestCase):
         self.factory = RequestFactory()
         self.user = User.objects.create_user(
             username='jacob', email='jacob@…', password='top_secret')
+        self.admin = User.objects.create_user(
+            username="admin", email="admin@...", password="Even More Secr3t",
+            is_staff=True,
+        )
 
     def test_form(self):
         atm = FormTest(autorun=False)
@@ -228,6 +232,11 @@ class FormTestCase(TestCase):
         response = views.TaskListView.as_view()(request)
         self.assertEqual(response.status_code, 200)
 
+        request = self.factory.get("/dashboard")
+        request.user = self.admin
+        response = views.TaskDashboardView.as_view()(request)
+        self.assertEqual(response.status_code, 200)
+
         atm.run()
         self.assertEqual(len(atm.form2.get_users_with_permission()), 0)
 
@@ -418,3 +427,22 @@ class ErrorTest(TestCase):
                          'Truth')
         self.assertEqual(atm._db.automationtaskmodel_set.all()[1].message,
                          "TypeError(\"'<' not supported between instances of 'bool' and 'datetime.datetime'\")")
+
+
+class SkipAutomation(flow.Automation):
+    start = Print("NOT SKIPPED").SkipIf(lambda x: False)
+    second = Print("SKIPPED").SkipIf(True).AsSoonAs(False)   # precedence of SkipIf over AsSoonAs
+    third = Print("Clearly printed")
+    forth = flow.Execute()  # Noop
+    end = flow.End()
+
+
+class SkipTest(TestCase):
+    def test_skipif(self):
+        with patch('sys.stdout', new=StringIO()) as fake_out:
+            atm = SkipAutomation()
+        output = fake_out.getvalue().splitlines()
+        self.assertTrue(atm.finished())
+        self.assertEqual(len(output), 2)
+        self.assertEqual(output[0], "start NOT SKIPPED")
+        self.assertEqual(output[-1], "third Clearly printed")
\ No newline at end of file
